
#include "App.h"
#include <iostream>


App::App(int argc, char **argv) : DVC_AppGL(argc, argv)
{
	bat = NULL;
	ball = NULL;
	goal = NULL;
	batController = NULL;
	ballController = NULL;
	goalController = NULL;
	useLogging = false;

	//these names must be the same ones used in the xml file
	batName = "bat";
	ballName = "ball";
	goalName = "goal";
}

App::~App()
{
	DeleteControllers();
	logFile.close();
}



bool App::ReloadScene()
{
	//call super class's ReloadScene method
	if ( !DVC_AppGL::ReloadScene() ) return false;
	DeleteControllers();
	if ( !FindBodies() ) return false;
	if ( !AddControllers() ) return false;

	//set surface angle to 90 degrees
	SetSurfaceAngle(90);

	return true;
}

// Set surface angle (degrees)
void App::SetSurfaceAngle(int angle)
{
	if (!ball || !ballController) return;

	if (angle == 90)
	{
		ball->SetSurfaceNormalForce(0);
		ballController->SetForceYComponent(ball->GetMass() * -9.81);
	}
	else
	{
		// WRITE CODE HERE!
		std::cout << "Not implemented yet..." << std::endl;
	}
}

bool App::Init()
{
	//Call the super class's Init method.
	//Be aware that the super class's Init method will
	//call ReloadScene
	if ( !DVC_AppGL::Init() )
	{
		std::cout << "Error, DVC_AppGL failed to initialize." << std::endl;
		return false;
	}

	//sanity check, make sure we are using the right xml file
	if ( prefs.get_string("/active_scene") != "hw1")
	{ 
		std::cout << "Error, pref active_scene != 'hw1'" << std::endl;
		return false;
	}

	//open the log file for writing data
	logFile.open("hw1_data.dat");

	return true;
}



bool App::AddControllers()
{
	//create and set a position controller to the bat
	//see the batPositionController definition in the header
	//file for explaination of the parameters.
	batController = new batPositionController( bat, ball, bat->GetQ()[0], bat->GetQ()[1], 2.0, 4.7, 0.0, 0.35);
	if ( !sim.SetBodyControllerByName( batName, *batController) )
	{
		std::cout << "Error adding position controller to body: " << batName << std::endl;
		delete batController;
		return false;
	}

	//create and set a position controller to the goal
	goalController = new goalPositionController( goal, goal->GetQ()[0], goal->GetQ()[1], goal->GetQ()[2]);
	if ( !sim.SetBodyControllerByName( goalName, *goalController) )
	{
		std::cout << "Error adding position controller to body: " << goalName << std::endl;
		delete goalController;
		return false;
	}

	//create and set a force controller for the ball
	ballController = new ballForceController();
	if ( !sim.SetBodyControllerByName( ballName, *ballController) )
	{
		std::cout << "Error adding force controller to body: " << ballName << std::endl;
		delete ballController;
		return false;
	}

	return true;	
}


// This method is called before each timestep
void App::PreStep()
{

	if (useLogging)
	{
		// Get position and velocity of ball
		const DVC::Vector<REAL>& q = ball->GetQ();
		const DVC::Vector<REAL>& nu = ball->GetNu();
		// Get current simulation time
		REAL time = sim.GetSimTime();

		// Write data to file.  The file will have 7 columns:
		// time position_x position_y position_theta velocity_x velocity_y velocity_theta
		logFile << time << " " << q[0] << " " << q[1] << " " << q[2] << " "
			<< nu[0] << " " << nu[1] << " " << nu[2] << std::endl;
	}
	
	/* Here is how you can see all the contacts
	*  from the previous step (including potential contacts)

	std::vector < DvcCollisionResult > contacts;
	sim.GetContactLocations( contacts );
	if ( !contacts.size() ) std::cout << "No contacts." << std::endl;
	for (unsigned int i = 0; i < contacts.size(); ++i)
	{
		DvcCollisionResult * collision = &contacts[i];
		std::cout << collision->b1->GetName() << " contacts with "
			<< collision->b2->GetName() << std::endl;
	}
	*/
	
}


//This method handles keyboard input
void App::KeyDown( char key )
{

	//First call super class's KeyDown method
	//Be careful, a key press could have an effect in super's method and this method
	DVC_AppGL::KeyDown(key); 


	switch (key)
	{
		case 'a':
			// When 'a' is pressed change the timestep to 0.0125
			prefs.set_double("/scenes/hw1/dynamics_settings/time_step", 0.0125);
			FlushSimSettings();
			break;

		case 'b':
			// When 'b' is pressed change the timestep to 0.0005
			prefs.set_double("/scenes/hw1/dynamics_settings/time_step", 0.0005);
			FlushSimSettings();
			break;

		default:
			break;
	}
}


void App::DeleteControllers()
{
	delete batController;
	delete ballController;
	delete goalController;
}

//Find the bat, ball and goal.
bool App::FindBodies()
{
	if ( !sim.GetBodyByName(batName, bat) )
	{
		std::cout << "Error finding bat in simulation." << std::endl;
		return false;
	}
	if ( !sim.GetBodyByName(ballName, ball) )
	{
		std::cout << "Error finding ball in simulation." << std::endl;
		return false;
	}
	if ( !sim.GetBodyByName(goalName, goal) )
	{
		std::cout << "Error finding goal in simulation."  << std::endl;
		return false;
	}
	return true;
}


