#pragma once

#include "DVC_AppGL.h"
#include "PositionController.h"
#include "ForceController.h"

#include <string>
#include <fstream>
#include <math.h>


class batPositionController; //forward declare batPositionController class
class goalPositionController; //forward declare goalPositionController class
class ballForceController; //forward declare ballForceController class

class App : public DVC_AppGL {

public:

	App(int argc=0, char **argv=0);	//constructor taking optional command line arguments
	~App();	//destructor


	bool Init();
	void PreStep();
	bool ReloadScene();
	void SetSurfaceAngle(int angle);
	void KeyDown( char key );

private:

	bool FindBodies();
	bool AddControllers();
	void DeleteControllers();

	bool useLogging;

	std::string batName;	//name in prefs xml file
	std::string ballName;	//name in prefs xml file
	std::string goalName;	//name in prefs xml file

	FixedBody * bat;	//pointer to bat
	FixedBody * goal;	//pointer to goal
	MovableBody * ball;	//pointer to ball
	
	batPositionController * batController;
	ballForceController * ballController;
	goalPositionController * goalController;
	
	std::ofstream logFile;

};


//a custom PositionController for the bat
class batPositionController : public PositionController {

public: 


	// @param bat pointer to position controlled body
	// @param ball pointer to movable body
	// @param x the x coordinate of body's center of gravity 
	// @param y the y coordinate of body's center of gravity
	// @param thetaStart the starting point of the body's rotation (radians)
	// @param thetaStop the end point of the body's rotation (radians)
	// @param startTime time when the bat begins rotation (seconds)
	// @param stopTime time when the bat ends rotation (seconds)
	batPositionController(FixedBody * bat, MovableBody * ball, REAL x, REAL y, REAL thetaStart, REAL thetaStop, REAL startTime, REAL stopTime) : PositionController(bat) {
		x_ = x;
		y_ = y;
		thetaStart_ = thetaStart;
		thetaStop_ = thetaStop;
		startTime_ = startTime;
		stopTime_ = stopTime;

		ball_ = ball;

		//check for errors
		if (startTime_ < 0) startTime_ = 0;
		if (stopTime_ < startTime_) stopTime_ = startTime_;
	}
	
	~batPositionController(){}

	// Note, this method is const which means it cannot modify
	// any of its member variables.  If you need to modify its
	// internal state you will need to write new methods and
	// call them from the App.
	// @param t the current simulation time (seconds)
	// @param outPos the output position vector
	void GetPosition(REAL t, DVC::Vector<REAL> & outPos) const {

		//These might be useful...
		//const DVC::Vector<REAL>& q = ball_->GetQ();
		//const DVC::Vector<REAL>& nu = ball_->GetNu();


		REAL theta;

		if (t < startTime_)
		{
			theta = thetaStart_;
		}
		else if (t > stopTime_)
		{
			theta = thetaStop_;
		}
		else
		{
			REAL dTime = t - startTime_;
			REAL dTheta = thetaStop_ - thetaStart_;
			theta = thetaStart_ + dTheta * dTime / (stopTime_ - startTime_);
		}

		
		outPos(0) = x_;	//set new x
		outPos(1) = y_;	//set new y
		outPos(2) = theta; //set new theta
	}

	REAL x_;
	REAL y_;
	REAL thetaStart_;
	REAL thetaStop_;
	REAL startTime_;
	REAL stopTime_;
	MovableBody * ball_;

};


//a custom PositionController for the goal
class goalPositionController : public PositionController {
public: 

	// @param body Pointer to position controlled body
	// @param x the x coordinate of body's center of gravity 
	// @param y the y coordinate of body's center of gravity
	// @param theta the theta coordinate of body's center of gravity (radians)
	goalPositionController(FixedBody * body, REAL x, REAL y, REAL theta) : PositionController(body) {
		x_ = x;
		y_ = y;
		theta_ = theta;
	}
	
	~goalPositionController(){}

	// Note, this method is const which means it cannot modify
	// any of its member variables.  If you need to modify its
	// internal state you will need to write new methods and
	// call them from the App.
	// @param t the current simulation time (seconds)
	// @param outPos the output position vector
	void GetPosition(REAL t, DVC::Vector<REAL> & outPos) const {
		outPos(0) = x_;
		outPos(1) = y_;
		outPos(2) = theta_;
	}

private:

	REAL x_;
	REAL y_;
	REAL theta_;
};


//a custom ForceController for the ball
class ballForceController : public ForceController {
public: 
	ballForceController()
	{ 
		outForceY = 0;
	}

	void SetForceYComponent(REAL y)
	{
		outForceY = y;
	}
	
	// @param t the currect simulation time
	// @param outForce the output force vector
	// @param pos body's current position x,y,theta
	// @param nu body's current velocity x,y,theta
	void GetForce(REAL t, DVC::Vector<REAL> & outForce, const DVC::Vector<REAL> & pos, const DVC::Vector<REAL> & nu) const {
		outForce(0) = 0;
		outForce(1) = outForceY;
		outForce(2) = 0;
	}

private:

	REAL outForceY;
	
};



